require 'test/unit'
require 'controlbot'

# Relevant agents must be started before running this test.
class TestControlbot < Test::Unit::TestCase

  def setup
    @agent = ControlBot.new
    name = @agent.send("what is your name?")
    @agent.send("#{name}: set delay to 5.0")
    @agent.send("#{name}: set sleep to 4.0")
  end

  def test_alice
    actual = @agent.send("Alice: if response =~ /JUDGE/ then score = 0 end")
    assert_match(/Okay/, actual)
  end

  def test_link
    actual = @agent.send('link: what is the subject of "I like traffic lights."')
    assert_match(/\bI\b/, actual)
    actual = @agent.send('link: what is the verb in: "I like traffic lights."')
    assert_match(/\blike\b/, actual)
  end

  def test_monty

    actual = @agent.send('monty: help')
    assert_match(/montylingua/i, actual)

    actual = @agent.send('monty: subject: John ate an apple.')
    assert_match(/John/, actual)

    actual = @agent.send('monty: what is the object in John ate an apple.')
    assert_match(/an apple/, actual)
  end

  def test_delay
    name = @agent.send("what is your name?")
    actual = @agent.send("#{name}: set the delay to 0.5")
    assert_match(/0\.5/, actual)
  end

  def test_sleep
    name = @agent.send("what is your name?")
    actual = @agent.send("#{name}: set sleep to 0.5")
    assert_match(/0\.5/, actual)
  end

  def test_ling
    actual = @agent.send('plural: goose')
    assert_match(/geese/, actual)

    actual = @agent.send('infinitive: begat')
    assert_match(/beget/, actual)
  end

  def test_isr
    actual = @agent.send('!sr In America, you can always find a party.')
    assert_match(/In Soviet Russia, the party can always find YOU!/, actual)
  end

  def test_bc
    actual = @agent.send("bcagent: add A recommends silver")
    assert_match(/Okay/, actual)

    actual = @agent.send("bcagent: if A recommends silver, then B buys silver")
    assert_match(/Okay/, actual)

    actual = @agent.send("bcagent: is 'B buys silver' true?")
    assert_match(/Yes/, actual)
  end

  def test_megahal
    actual = @agent.send("megahal: help")
    assert_match(/Jason Hutchens/, actual)
  end

  def test_wnbot
    actual = @agent.send("wnbot: help")
    assert_match(/Wordnet/i, actual)
  end

  def test_gagent
    actual = @agent.send("gagent: help")
    assert_match(/logic/, actual)
  end

end
